//
//  ViewController.m
//  Beacon-CL
//
//  Created by Matthias Krauß on 22.09.14.
//  Copyright (c) 2014 Matthias Krauß. All rights reserved.
//

#import "ViewController.h"
#import <CoreLocation/CoreLocation.h>

#define EXPLODE_IN_PRODUCTION_CODE 1

//Estimote
#define MY_BEACON_UUID   @"B9407F30-F5F8-466E-AFF9-25556B57FE6D"

//PAK
//#define MY_BEACON_UUID @"16A2230E-0B8B-4D3D-BF83-36DA12001990"

@interface ViewController () <CLLocationManagerDelegate, UITableViewDataSource, UITableViewDelegate>

@property (strong) CLLocationManager* locationManager;
@property (strong) NSMutableArray* entries;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.entries = [NSMutableArray array];
    self.locationManager = [CLLocationManager new];
    if (!self.locationManager) {
        NSLog(@"Location Services init failed");
        exit(0);
    }
    self.locationManager.delegate = self;

    self.label.text = @"Checking authorization";
    if (![self checkAuthorization]) {
        [self.locationManager requestWhenInUseAuthorization];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (BOOL) checkAuthorization {
    CLAuthorizationStatus status = [CLLocationManager authorizationStatus];
    if ((status == kCLAuthorizationStatusAuthorizedAlways) || (status == kCLAuthorizationStatusAuthorizedWhenInUse)) {
        self.label.text = @"Starting to range beacons";
        NSUUID* beaconUUID = [[NSUUID alloc] initWithUUIDString:MY_BEACON_UUID];
        CLBeaconRegion* region = [[CLBeaconRegion alloc]
                       initWithProximityUUID:beaconUUID
                       identifier:MY_BEACON_UUID];

        [self.locationManager startRangingBeaconsInRegion:region];
        return YES;
    } else return NO;
}

- (void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status {
    [self checkAuthorization];
}

- (void)locationManager:(CLLocationManager *)manager didRangeBeacons:(NSArray *)beacons inRegion:(CLBeaconRegion *)region {
    NSMutableArray* newEntries = [NSMutableArray array];
    for (CLBeacon* beacon in beacons) {
        NSString* prox = @"?";
        switch (beacon.proximity) {
            case CLProximityUnknown: prox = @"unknown"; break;
            case CLProximityImmediate: prox = @"immediate"; break;
            case CLProximityNear: prox = @"near"; break;
            case CLProximityFar: prox = @"far"; break;
        }
        NSString* entry = [NSString stringWithFormat:@"Major %i minor %i\nRSSI %i proximity %@ ",
                           beacon.major.intValue,
                           beacon.minor.intValue,
                           (int)(beacon.rssi),
                           prox];
        [newEntries addObject:entry];
    }
    self.entries = [newEntries copy];
    [self.table reloadData];
    if (beacons.count > 0) self.label.text = [NSString stringWithFormat:@"Found %i beacons",(int)(beacons.count)];
    else self.label.text = @"No beacons in range";
}

- (void)locationManager:(CLLocationManager *)manager rangingBeaconsDidFailForRegion:(CLBeaconRegion *)region withError:(NSError *)error {
    self.label.text = [NSString stringWithFormat:@"Ranging failed: %@",[error localizedDescription]];
}

//UITableView delegate and data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.entries.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier];
        cell.textLabel.numberOfLines = 2;
        cell.textLabel.lineBreakMode = NSLineBreakByWordWrapping;
    }
    
    cell.textLabel.text = self.entries[indexPath.row];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 50;
}

@end
